/*
 * AddressBookDump
 *
 * Author: Matthew Might
 * Site:   http://matt.might.net/
 *
 * A program for dumping out an address
 * book as a sequence of S-Expressions.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <AddressBook/ABAddressBookC.h>
#include <AddressBook/ABGlobalsC.h>
#include <CoreFoundation/CoreFoundation.h>

// The global address book:
ABAddressBookRef AB ;

// Convert a Core Foundation string 
// to a null-terminated C string:
const char* cstring (CFStringRef s) {
  return (const char*)
  CFStringGetCStringPtr(s, 
                        kCFStringEncodingMacRoman) ;
}

int main (int argc, const char* argv[]) {
  	
  // By default, choose the group Family:
  const char* groupName = "Family" ;
  
  // Usage: $0 ["<group name>"]
  // Ex: $0 "Holiday Card List"
  if (argc >= 2)
    groupName = argv[1] ;
	
  CFStringRef cfGroupName 
  = CFStringCreateWithBytes(NULL,
                            (UInt8*)groupName,
                            strlen(groupName),
                            kCFStringEncodingMacRoman,
                            false);
	
  // Find that group:
  ABSearchElementRef matchGroupName 
  = ABGroupCreateSearchElement(kABGroupNameProperty,
                               NULL,
                               NULL,
                               cfGroupName,
                               kABEqualCaseInsensitive);
  
  // Grab the global, shared address book:
  AB = ABGetSharedAddressBook();
  
  // All groups with that name:
  CFArrayRef groupsFound 
  = ABCopyArrayOfMatchingRecords(AB, matchGroupName) ;
	
  if (CFArrayGetCount(groupsFound) == 0) {
    printf("error: couldn't find address book group '%s'\n",
           groupName);
    exit(-1) ;
  }
  
	ABGroupRef matchedGroup 
  = (ABGroupRef)CFArrayGetValueAtIndex(groupsFound, 0);
  
  // All people of the matched group:
  CFArrayRef members
  = ABGroupCopyArrayOfAllMembers(matchedGroup) ;
  
  // Total number of people:
	int len = CFArrayGetCount(members) ;  
  
  // Give each entry a unique id:
  int uid = 0 ;
  
  // Iterate over all people:
  for (int i = 0; i < len; ++i) {
    
    ABPersonRef person 
    = (ABPersonRef)CFArrayGetValueAtIndex(members, i) ;
    
    // Grab properties of each person:
    CFStringRef firstName 
    = ABRecordCopyValue(person, kABFirstNameProperty);
    
    CFStringRef lastName 
    = ABRecordCopyValue(person, kABLastNameProperty);
    
    ABMultiValueRef address
    = ABRecordCopyValue(person, kABAddressProperty);
    
    ABMultiValueRef relatives
    = ABRecordCopyValue(person, kABRelatedNamesProperty);
    
    CFStringRef title
    = ABRecordCopyValue(person, kABTitleProperty);
    
    // Print properties of each person:
    printf("((first-name \"%s\")\n", cstring(firstName)) ;
    printf(" (last-name  \"%s\")\n", cstring(lastName)) ;
    
    if (title)
      printf(" (title      \"%s\")\n", cstring(title)) ;
    
    // There could be multiple address.
    // We have to loop through all of them.
    int numAddresses = ABMultiValueCount(address) ;
    for (int k = 0; k < numAddresses; ++k) {
      
      // Check the type:
      CFStringRef addrType
      = ABMultiValueCopyLabelAtIndex(address, k) ;
      
      // Keep searching if it's not the home address:
      if (CFStringCompare(addrType,kABHomeLabel,0) != 0) 
        continue ;
      
      printf(" (address    ") ;
      
      // Addresses have multiple keys:
      // Street, State, ZIP, Country, City
      CFDictionaryRef location 
      = ABMultiValueCopyValueAtIndex(address, k) ;
      
      int size = CFDictionaryGetCount(location) ;
      
      CFStringRef keys[size] ;
      CFStringRef values[size] ;
      
      CFDictionaryGetKeysAndValues(location, 
                                   (const void**)&keys,
                                   (const void**)&values) ;
      
      for (int a = 0; a < size; ++a) {
        if (a)
          printf("             ") ;
        
        printf("(%-12s \"%s\")", 
               cstring(keys[a]), 
               cstring(values[a])) ;
        if (a+1 == size) 
          printf(")");
        
        printf("\n");
      }
    }
    
    // We have to search all relatives for the spouse/partner:
    int numRelatives = ABMultiValueCount(relatives) ;
    for (int j = 0; j < numRelatives; ++j) {
      CFStringRef relativeType
      = ABMultiValueCopyLabelAtIndex(relatives, j) ;
      
      CFStringRef relativeName 
      = ABMultiValueCopyValueAtIndex(relatives, j) ;
      
      if (!CFStringCompare(relativeType,kABSpouseLabel,0)) 
        printf(" (spouse     \"%s\")\n",
               cstring(relativeName)) ;
      
      if (!CFStringCompare(relativeType,kABPartnerLabel,0))
        printf(" (partner    \"%s\")\n",
               cstring(relativeName)) ;
    }
    printf(" (uid        %i))\n\n",     uid++) ;
  }
  return EXIT_SUCCESS;
}
